import { styled } from "solid-styled-components";
import { vector2d } from "../vector";
import { createMemo, createSignal } from "solid-js";
import { useTimer } from "../providers/timer";
import { checkElementOverlap } from "./collision";

import "./crystal.css";

export default function Crystal(props: { size?: number; position?: vector2d }) {
  const size = props.size ?? 100;

  // Position

  const [x] = createSignal(
    props.position?.x ? props.position?.x - size / 2 : 0
  );
  const [y] = createSignal(
    props.position?.y ? props.position?.y - size / 2 : 0
  );

  // Energy

  const [energy, setEnergy] = createSignal(100);
  const [maxEnergy, setMaxEnergy] = createSignal(100);

  const rechargeProgress = createMemo(() => (energy() / maxEnergy()) * 100);

  const isAlive = createMemo(() => energy() > 0);

  function checkAnyPlayerUnderCrystal() {
    const players = document.querySelectorAll<HTMLElement>(".player");
    const crystal = document.querySelector<HTMLElement>(".crystal");

    for (const player of players) {
      if (crystal && checkElementOverlap(crystal, player)) {
        return true;
      }
    }
    return false;
  }

  function updateEnergy() {
    const doRecharge = checkAnyPlayerUnderCrystal();
    if (doRecharge) {
      if (energy() < maxEnergy()) {
        setEnergy(energy() + 1);
      }
    } else {
      if (energy() > 0) {
        setEnergy(energy() - 1);
      }
    }
  }

  const [{ subscribe }] = useTimer();
  subscribe(() => updateEnergy());

  // Renderer

  const renderRandomFactor = Math.random();

  const CrystalElement = styled("svg")`
    width: ${size.toString()}px;
    height: ${size.toString()}px;

    background-color: transparent;
    position: absolute;

    --half-size: calc(var(--size) / 2);
    --stroke-width: 8px;
    --radius: calc(
      (var(--size) - var(--stroke-width)) / 2 - var(--stroke-width) / 2
    );
    --circumference: calc(var(--radius) * pi * 2);
    --dash: calc((var(--progress) * var(--circumference)) / 100);
  `;

  const CrystalCircleActive = styled("circle")`
    cx: var(--half-size);
    cy: var(--half-size);
    r: var(--radius);
    stroke-width: var(--stroke-width);
    fill: none;
    stroke-linecap: round;

    animation: alternate 10s crystal--fill-wabble infinite ease-in-out;

    transform-origin: var(--half-size) var(--half-size);
    stroke-dasharray: var(--dash) calc(var(--circumference) - var(--dash));
    transition: all 0.3s linear 0s;
    stroke: var(--color-primary);
  `;

  const CrystalCircleInactive = styled("circle")`
    cx: var(--half-size);
    cy: var(--half-size);
    r: var(--radius);
    stroke-width: var(--stroke-width);
    fill: none;
    stroke-linecap: round;

    stroke: #ddd;
  `;

  return (
    <CrystalElement
      class={`crystal ${isAlive() ? `crystal--powerful` : `crystal--powerless`}`}
      style={{
        left: `${x()}px`,
        top: `${y()}px`,

        "--size": `${size}px`,
        "--rotation-factor": renderRandomFactor * 10,
        "--progress": rechargeProgress(),
      }}
    >
      <CrystalCircleInactive />
      <CrystalCircleActive class="crystal--energy-indicator" />
    </CrystalElement>
  );
}
